#
# Graphical user interface for the ConnectFour class
#
import turtle
import connect_four

# Size of each cell in the grid
cell_size = 50

# Colors for the two players
colors = ("red", "blue")

# The connect four game instance
game = None

# Window
wn = None

# Turtle for drawing the tokens
t = None

# Turtle for drawing the text
text_turtle = None

# upper left corner of grid, determined from window size
start_x = None
start_y = None


def draw_grid(num_rows, num_cols):
    '''
    Draws an empty grid of the given size.
    '''
    t.setheading(270) # south
    #t.right(90)
    for col in range(num_cols + 1):
        t.penup()
        t.goto(start_x + col * cell_size, start_y)       
        t.pendown()    
        t.forward(cell_size * num_rows)
    #t.left(90)
    t.setheading(0) # east
    for row in range(num_rows + 1):
        t.penup()
        t.goto(start_x, start_y - row * cell_size)       
        t.pendown()    
        t.forward(cell_size * num_cols)

    
def print_message(msg):
    '''
    Prints the given text at the top of the grid.
    '''
    text_turtle.hideturtle()
    text_turtle.clear()
    text_turtle.penup()
    text_turtle.goto(start_x, start_y)
    text_turtle.write(msg, font=("Ariel", 16, "normal"))    


def do_play(col):
    '''
    Make a move in the given column and draw a token.
    '''
    color = colors[game.whose_turn()]            
    row = game.play(col)
    if row >= 0:
        t.pencolor(color)
        t.fillcolor(color)
        t.hideturtle()
        t.penup()
        t.goto(start_x + col * cell_size + cell_size / 2, start_y + cell_size / 2)
        t.setheading(270) # south
        t.showturtle()           
        t.forward((row + 1) * cell_size)
        t.stamp()
        t.hideturtle()
        if game.is_over():
            print_message(color + " has won!")
        else:
            next_turn = colors[game.whose_turn()]
            print_message("It is " + next_turn + "'s turn. Click in a column.")


def click_handler(mouse_x, mouse_y):
    '''
    Invoked when the mouse is clicked in the window.
    '''
    # if the mouse is clicked within the left and right of the grid,
    # then figure out the row and call do_play
    num_cols = len(game.get_grid()[0])
    if mouse_x >= start_x and mouse_x <= start_x + cell_size * num_cols:
        col = int((mouse_x - start_x) // cell_size)
        do_play(col)
    else:
        # Otherwise, exit.
        # Note that if you are running from Wing, you'll still need to
        # restart the shell to completely exit
        wn.bye() 
  
def start(num_rows, num_cols, run_length):
    '''
    Set up the graphics and start a game.
    '''
    # initialize everything
    global game, wn, t, text_turtle, start_x, start_y
    wn = turtle.Screen()
    start_x = -wn.window_width() / 2 + cell_size
    start_y = wn.window_height() / 2 - cell_size    
    t = turtle.Turtle()
    text_turtle = turtle.Turtle()
    game = connect_four.ConnectFour(num_rows, num_cols, run_length)
    
    # draw the empty grid
    t.speed(0)    
    draw_grid(num_rows, num_cols)
    
    # set up turtle for drawing tokens
    t.hideturtle()
    t.shape("circle")
    t.speed(3)
    t.penup() 
    
    # print the first message using the "text" turtle
    next_turn = colors[game.whose_turn()]
    print_message("It is " + next_turn + "'s turn. Click in a column. ")
    
    # set up the click_handler function to execute when mouse is clicked,
    # and wait for mouse clicks
    wn.onclick(click_handler)
    wn.mainloop()


if __name__ == "__main__":
    start(6, 7, 4)